#include "ppc.h"
#include "framebuffer.h"
#include <math.h>

PPC::PPC(int _w, int _h, float hfov) : w(_w), h(_h) {

  C = V3(0.0f, 0.0f, 0.0f);
  a = V3(1.0f, 0.0f, 0.0f);
  b = V3(0.0f, -1.0f, 0.0f);
  c[0] = -(float)w/2.0f;
  c[1] = +(float)h/2.0f;
  float tan2 = tanf(hfov/180.0f*3.1415f/2.0f);
  c[2] = -(float)w/2.0f/tan2;

}

bool PPC::Project(V3 P, V3 &projP) {

  M3x3 camM;
  camM.SetColumn(0, a);
  camM.SetColumn(1, b);
  camM.SetColumn(2, c);
  M3x3 invCamM = camM.Invert();
  V3 Q = invCamM*(P-C);

  if (Q[2] < 0.0f)
    return false; // behind head

  projP[0] = Q[0]/Q[2];
  projP[1] = Q[1]/Q[2];
  projP[2] = Q[2];

  return true;

}


void PPC::Visualize(PPC *ppc, FrameBuffer *fb) {

  int psize = 5;
  unsigned int color = 0xFF000000;
  V3 colorv; colorv.SetFromColor(color);
  fb->DrawPoint3D(C, psize, color, ppc);
  fb->DrawSegment3D(C, C+c, colorv, colorv, ppc);
  fb->DrawSegment3D(C+c, C+c+a*(float)w, colorv, colorv, ppc);
  fb->DrawSegment3D(C+c+a*(float)w, C+c+a*(float)w+b*(float)h, 
    colorv, colorv, ppc);
  fb->DrawSegment3D(C+c+a*(float)w+b*(float)h, 
    C+c+b*(float)h, 
    colorv, colorv, ppc);
  fb->DrawSegment3D(C+c+b*(float)h, C+c,
    colorv, colorv, ppc);

}

void PPC::Translate(V3 trans) {

  C = C+trans;

}

void PPC::TranslateFW(float step) {

  V3 vd = GetVD();
  Translate(vd*step);

}

V3 PPC::GetVD() {

  V3 ret = a^b;
  ret = ret.Normalized();
  return ret;

}