#include "triangle.h"
#include "m3x3.h"

void Triangle::Rasterize(FrameBuffer *fb) {

  AABB bb;
  bb.AddPoint(verts[0]);
  bb.AddPoint(verts[1]);
  bb.AddPoint(verts[2]);

  if (!fb->ClipBB(bb))
    return;

  int lefti = (int) (bb[0][0]+0.5f);
  int righti = (int) (bb[1][0]-0.5f);
  int topi = (int) (bb[0][1]+0.5f);
  int bottomi = (int) (bb[1][1]-0.5f);

  M3x3 eeqs;
  eeqs[0] = SetEdgeEquation(0);
  eeqs[1] = SetEdgeEquation(1);
  eeqs[2] = SetEdgeEquation(2);
 
  for (int v = topi; v <= bottomi; v++) {
    for (int u = lefti; u <= righti; u++) {
      V3 pixv(.5f+(float)u, .5f+(float)v, 1.0f);
      V3 sdd = eeqs*pixv;
      if (sdd[0] < 0.0f || sdd[1] < 0.0f || 
        sdd[2] < 0.0f)
        continue;
      unsigned int colui = color.GetColor();
      fb->Set(u, v, colui);
    }
  }

}

V3 Triangle::SetEdgeEquation(int i) {

  V3 ret;
  V3 P0 = verts[i];
  V3 P1 = verts[(i+1)%3];
  ret[0] = P1[1]-P0[1];
  ret[1] = -P1[0]+P0[0];
  ret[2] = -P0[0]*ret[0]-P0[1]*ret[1];

  V3 P2 = verts[(i+2)%3];
  P2[2] = 1.0f;
  if (P2*ret < 0.0f)
    ret = ret * -1.0f;

  return ret;

}

void Triangle::DrawWireFrame(FrameBuffer *fb) {

  for (int i = 0; i < 3; i++) {
    int _i = (i+1)%3;
    fb->DrawSegment(verts[i], verts[_i], colors[i], colors[_i]);
  }

}